/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "param.h"
#include <signal.h>

static pid_t pid_to_kill = 0;

param *local_p;

/* handler used to catch various signals.
 *    also handles ALARM signal if a test times out
 *    */
static void fault_handler(int signo)
{
	char *signalstr = strsignal(signo);
	if (signo == SIGALRM)
		signalstr = "Timed Out";
	fprintf(stdout, "%-2d : %-23s : %-100s : %s : %-3d : %s\n",
		local_p->test_number, local_p->test_value,
		local_p->test_name, "FAIL", signo, signalstr);

	if (pid_to_kill != 0) {
		kill(pid_to_kill, SIGTERM);
	}
	exit(128 + signo);
}				/* fault_handler */

/* initializes fault handler */
static int setup_fault_catcher(param * p, pid_t kill_this_pid)
{
	local_p = p;
	struct sigaction act_params;

	pid_to_kill = kill_this_pid;

	act_params.sa_handler = fault_handler;
	sigfillset(&act_params.sa_mask);
	act_params.sa_flags = 0;

	/* set up handler for "Segmentation violation" */
	if (sigaction(SIGSEGV, &act_params, NULL) != 0) {
		perror("sigaction SIGSEGV");
		exs_close(p->fd, EXS_BLOCK | EXS_DONTLINGER, NULL, NULL);
		return -1;
	}
	/* set up handler for "Abort" */
	if (sigaction(SIGABRT, &act_params, NULL) != 0) {
		perror("sigaction SIGABRT");
		exs_close(p->fd, EXS_BLOCK | EXS_DONTLINGER, NULL, NULL);
		return -1;
	}
	/* set up handler for "Alarm clock" */
	if (sigaction(SIGALRM, &act_params, NULL) != 0) {
		fprintf(stdout, "ALARMAMAMAM");
		perror("sigaction SIGALRM");
		exs_close(p->fd, EXS_BLOCK | EXS_DONTLINGER, NULL, NULL);
		return -1;
	}

	return 0;
}				/* setup_fault_catcher */
