/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_BLOCKING_SENDFILE_SETUP_H
#define _EXS_BLOCKING_SENDFILE_SETUP_H

#include "../result.h"

#define BLOCKING_SENDFILE_TESTS 16

#define EXS_MAX_SENDFILE_SIZE 1073741824

static char *all_blocking_sendfile_tests[BLOCKING_SENDFILE_TESTS] = {
	"1  : EPERM                   : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [no exs_init]",
	"2  : ENOTCONN                : exs_blocking_sendfile(NOT_CONNECTED_FD, sendvec, sendvec_cnt, flags)",
	"3  : EBADF                   : exs_blocking_sendfile(BAD_FD, sendvec, sendvec_cnt, flags)",
	"4  : ENODEV                  : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [fd cannot be mmapped]",
	"5  : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [bad exs_fileds]",
	"6  : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [bad exs_path]",
	"7  : EFBIG                   : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [file size > 1GB]",
	"8  : EINVAL                  : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [offset + length > file size]",
	"9  : EINVAL                  : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [offset > file size]",
	"10 : EINVAL                  : exs_blocking_sendfile(fd, sendvec, BAD_SENDVEC_CNT, flags, qh, ah)",
	"11 : EOPNOTSUPP              : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, BAD_FLAGS)",
	"12 : EINVAL                  : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, EXS_UNSIGNALED & EXS_BLOCK)",
	"13 : EPIPE                   : exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [gracefull shut down for writing]",
	"14 : OK_EXS_BLOCKING_SENDFILE: exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags)",
	"15 : OK_EXS_BLOCKING_SENDFILE: exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [send the same file twice]",
	"16 : OK_EXS_BLOCKING_SENDFILE: exs_blocking_sendfile(fd, sendvec, sendvec_cnt, flags) [using exs_pathvec]"
};

static int list_blocking_sendfile_tests(int select)
{
	if (select < 0 || select > BLOCKING_SENDFILE_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			BLOCKING_SENDFILE_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < BLOCKING_SENDFILE_TESTS; i++) {
			fprintf(stdout, "%s\n",
				all_blocking_sendfile_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n",
			all_blocking_sendfile_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_blocking_sendfile(param * p)
{
	char *name = "ok_exs_blocking_sendfile";

	p->file_length = 1024;

	exs_xferfile_t sendvec;
	memset(&sendvec, 0, sizeof(exs_xferfile_t));

	int fd = open(exs_test_small_file, O_RDONLY);

	sendvec.exs_xfv.exs_fdv.exs_fildes = fd;
	sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
	sendvec.exs_xfv.exs_fdv.exs_length = (size_t) 0;	//go until EOF
	sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

	p->sendvec = &sendvec;
	p->sendvec->exs_xfv_type = EXS_FDVEC;

	p->sendvec_cnt = 1;

	int ret =
	    exs_blocking_sendfile(p->fd, p->sendvec, p->sendvec_cnt,
				  p->flags);

	close(fd);

	return ret;
}
#endif
