/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_CONNECT_SETUP_H
#define _EXS_CONNECT_SETUP_H

#include "../result.h"

#define CONNECT_TESTS 5

static char *all_connect_tests[CONNECT_TESTS] = {
	"1  : EPERM                   : exs_connect(fd, ai_addr, ai_addrlen, flags, to, qh, ah ) [no exs_init]",
	"2  : EBADF                   : exs_connect(BAD_FD, ai_addr, ai_addrlen, flags, to, qh, ah)",
	"3  : EINVAL                  : exs_connect(CONNECTED_FD, ai_addr, ai_addrlen, flags, to, qh, ah)",
	"4  : EINVAL                  : exs_connect(fd, ai_addr, ai_addrlen, BAD_FLAG, to, qh, ah)",
	"5  : OK_EXS_CONNECT          : exs_connect(fd, ai_addr, ai_addrlen, flags, to, qh, ah)"
};

static int list_connect_tests(int select)
{
	if (select < 0 || select > CONNECT_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			CONNECT_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < CONNECT_TESTS; i++) {
			fprintf(stdout, "%s\n", all_connect_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_connect_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_connect(param * p)
{
	char *name = "ok_exs_connect";

	struct addrinfo *info;
	struct addrinfo hints;
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = p->family;
	hints.ai_socktype = p->socktype;
	hints.ai_protocol = p->protocol;
	hints.ai_flags = AI_PASSIVE;

	int r = getaddrinfo(p->addr, p->port, &hints, &info);

	int ret =
	    exs_connect(p->fd, info->ai_addr, info->ai_addrlen, p->flags,
			p->to, p->qh, p->ah);

	if (ret < 0) {
		print_error(name);
		return -1;
	}
	return 0;
}

#endif
