/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_MDEREGISTER_SETUP_H
#define _EXS_MDEREGISTER_SETUP_H

#include "../result.h"

#define MDEREGISTER_TESTS 6

static char *all_mderegister_tests[MDEREGISTER_TESTS] = {
	"1  : EPERM                   : exs_mderegister(mh, 0) [no exs_init]",
	"2  : EINVAL                  : exs_mderegister(NULL, 0)",
	"3  : EINVAL                  : exs_mderegister(EXS_MHANDLE_INVALID, 0)",
	"4  : EINVAL                  : exs_mderegister(EXS_MHANDLE_UNREGISTERED, 0)",
	"5  : EINVAL                  : exs_mderegister(mh, BAD_FLAG)",
	"6  : OK_EXS_MDEREGISTER      : exs_mderegister(mh, 0)"
};

static int list_mderegister_tests(int select)
{
	if (select < 0 || select > MDEREGISTER_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			MDEREGISTER_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < MDEREGISTER_TESTS; i++) {
			fprintf(stdout, "%s\n", all_mderegister_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_mderegister_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_mderegister(param * p)
{
	char *name = "ok_exs_mderegister";

	int send_ret = exs_mderegister(p->send_mh, 0);
	int recv_ret = exs_mderegister(p->recv_mh, 0);

	if (send_ret == -1 || recv_ret == -1) {
		print_error(name);
		return -1;
	}

	return 0;
}

#endif
