/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_QSTATUS_SETUP_H
#define _EXS_QSTATUS_SETUP_H

#include "../result.h"

#define QSTATUS_TESTS 6

static char *all_qstatus_tests[QSTATUS_TESTS] = {
	"1  : EPERM                   : exs_qstatus(EXS_QATTR_WAIT) [no exs_init]",
	"2  : EINVAL                  : exs_qstatus(INVALID_QHANDLE, EXS_QATTR_WAIT, attr_value, attr_length)",
	"3  : EINVAL                  : exs_qstatus(qh,INVALID_ATTR_TYPE, attr_value, attr_length)",
	"4  : EINVAL                  : exs_qstatus(qh, EXS_QATTR_WAIT, attr_value, 0) [attr_length does not match]",
	"5  : EINVAL                  : exs_qstatus(qh, EXS_QATTR_WAIT, INVALID_ATTR_VALUE, attr_length)",
	"6  : OK_EXS_QSTATUS          : exs_qstatus(EXS_QATTR_WAIT)"
};

static int list_qstatus_tests(int select)
{
	if (select < 0 || select > QSTATUS_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			QSTATUS_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < QSTATUS_TESTS; i++) {
			if (strcmp(all_qstatus_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_qstatus_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_qstatus_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_qstatus(param * p)
{
	char *name = "ok_exs_qstatus";

	int *buf;
	buf = malloc(sizeof(int));
	p->attr_value = buf;

	int ret =
	    exs_qstatus(p->qh, EXS_QATTR_WAIT, p->attr_value, sizeof(int));

	if (ret < 0) {
		print_error(name);
		return -1;
	}
	return 0;
}

#endif
