/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_RECV_SETUP_H
#define _EXS_RECV_SETUP_H

#include "../result.h"

#define RECV_TESTS 10

static char *all_recv_tests[RECV_TESTS] = {
	"1  : EPERM                   : exs_recv(fd, recv_buf, RECV_BUFSIZE, flags, qh, ah, mh) [no exs_init]",
	"2  : EBADF                   : exs_recv(BAD_FD, recv_buf, RECV_BUFSIZE, flags, qh, ah, mh)",
	"3  : ENOTCONN                : exs_recv(NOT_CONNECTED_FD, recv_buf, RECV_BUFSIZE, flags, qh, ah, mh)",
	"4  : EINVAL                  : exs_recv(fd, BAD_BUFFER, RECV_BUFSIZE, flags, qh, ah, mh)",
	"5  : EINVAL                  : exs_recv(fd, recv_buf, ZERO_LENGTH, flags, qh, ah, mh) [length < 1 ]",
	"6  : EINVAL                  : exs_recv(fd, recv_buf, MAX_LENGTH+1, flags, qh, ah, mh) [length > RDMA channel]",
	"7  : EOPNOTSUPP              : exs_recv(fd, recv_buf, RECV_BUFSIZE, BAD_FLAG, qh, ah, mh)",
	"8  : EINVAL                  : exs_recv(fd, recv_buf, RECV_BUFSIZE, flags, INVALID_QHANDLE, ah, mh)",
	"9  : EINVAL                  : exs_recv(fd, recv_buf, RECV_BUFSIZE, 0, INVALID_QHANDLE, ah, mh)",
	"10 : OK_EXS_RECV             : exs_recv(fd, recv_buf, RECV_BUFSIZE, flags, qh, ah, mh)",
};

static int list_recv_tests(int select)
{
	if (select < 0 || select > RECV_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			RECV_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < RECV_TESTS; i++) {
			if (strcmp(all_recv_tests[i], "") != 0)
				fprintf(stdout, "%s\n", all_recv_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_recv_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_recv(param * p)
{
	char *name = "ok_exs_recv";

	int ret =
	    exs_recv(p->recv_fd, p->recv_buf, p->RECV_BUFSIZE, p->flags,
		     p->qh, p->ah, p->mh);

	if (p->mode == 0) {
		if (ret < p->RECV_BUFSIZE) {
			print_error(name);
			return -1;
		}
	}

	if (p->mode > 0) {
		if (ret < 0) {
			print_error(name);
			return -1;
		}
	}

	return ret;

}

#endif
