/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_SEND_SETUP_H
#define _EXS_SEND_SETUP_H

#include "../result.h"

#define SEND_TESTS 16

static char *all_send_tests[SEND_TESTS] = {
	"1  : EPERM                   : exs_send(fd, send_buf, SEND_BUFSIZE, flags, qh, ah, mh) [no exs_init]",
	"2  : ENOTCONN                : exs_send(NOT_CONNECTED_FD, send_buf, SEND_BUFSIZE, flags, qh, ah, mh)",
	"3  : EBADF                   : exs_send(BAD_FD, send_buf, SEND_BUFSIZE, flags, qh, ah, mh)",
	"4  : EINVAL                  : exs_send(fd, NULL_BUFFER, SEND_BUFSIZE, flags, qh, ah, mh)",
	"5  : EINVAL                  : exs_send(fd, send_buf, SEND_BUFSIZE, EXS_UNSIGNALED & EXS_BLOCK, qh, ah, mh)",
	"6  : EOPNOTSUPP              : exs_send(fd, send_buf, SEND_BUFSIZE, BAD_FLAG, qh, ah, mh)",
	"7  : EINVAL                  : exs_send(fd, send_buf, SEND_BUFSIZE, flags, INVALID_QHANDLE, ah, mh)",
	"8  : EINVAL                  : exs_send(fd, send_buf, SEND_BUFSIZE, 0, INVALID_QHANDLE, ah, mh)",
	"",
	"10 : EPIPE                   : exs_send(p->fd, send_buf, SEND_BUFSIZE, flags, qh, ah, mh) [gracefull shut down for writing]",
	"",
	"",
	"",
	"",
	"",
	"16 : OK_EXS_SEND             : exs_send(p->fd, send_buf, SEND_BUFSIZE, flags, qh, ah, mh)"
};

static int list_send_tests(int select)
{
	if (select < 0 || select > SEND_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			SEND_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < SEND_TESTS; i++) {
			if (strcmp(all_send_tests[i], "") != 0)
				fprintf(stdout, "%s\n", all_send_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_send_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_send(param * p)
{
	char *name = "ok_exs_send";

	int ret =
	    exs_send(p->fd, p->send_buf, p->SEND_BUFSIZE, p->flags, p->qh,
		     p->ah, p->mh);

	if (p->mode == 0) {
		if (ret < p->SEND_BUFSIZE) {
			print_error(name);
			return -1;
		}
	}

	if (p->mode > 0) {
		if (ret < 0) {
			print_error(name);
			return -1;
		}
	}
	return ret;

}

#endif
