/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_SENDFILE_SETUP_H
#define _EXS_SENDFILE_SETUP_H

#include "../result.h"

#define SENDFILE_TESTS 17

#define EXS_MAX_SENDFILE_SIZE 1073741824

static char *all_sendfile_tests[SENDFILE_TESTS] = {
	"1  : EPERM                   : exs_sendfile(fd, sendvec, sendvec_cnt, flags ,qh, ah) [no exs_init]",
	"2  : ENOTCONN                : exs_sendfile(NOT_CONNECTED_FD, sendvec, sendvec_cnt, flags, qh, ah)",
	"3  : EBADF                   : exs_sendfile(BAD_FD, sendvec, sendvec_cnt, flags, qh, ah)",
	"4  : ENODEV                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [fd cannot be mmapped]",
	"5  : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [bad exs_fileds]",
	"6  : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [bad exs_path]",
	"7  : EFBIG                   : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [file size > 1GB]",
	"8  : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [offset + length > file size]",
	"9  : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [offset > file size]",
	"10 : EINVAL                  : exs_sendfile(fd, sendvec, BAD_SENDVEC_CNT, flags, qh, ah)",
	"11 : EOPNOTSUPP              : exs_sendfile(fd, sendvec, sendvec_cnt, BAD_FLAGS, qh, ah)",
	"12 : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, EXS_UNSIGNALED & EXS_BLOCK, qh, ah)",
	"13 : EINVAL                  : exs_sendfile(fd, sendvec, sendvec_cnt, EXS_BLOCK, INVALID_QHANDLE, ah)",
	"14 : EPIPE                   : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [gracefull shut down for writing]",
	"15 : OK_EXS_SENDFILE         : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah)",
	"16 : OK_EXS_SENDFILE         : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [send the same file twice]",
	"17 : OK_EXS_SENDFILE         : exs_sendfile(fd, sendvec, sendvec_cnt, flags, qh, ah) [using exs_pathvec]",
};

static int list_sendfile_tests(int select)
{
	if (select < 0 || select > SENDFILE_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			SENDFILE_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < SENDFILE_TESTS; i++) {
			if (strcmp(all_sendfile_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_sendfile_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_sendfile_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_sendfile(param * p)
{
	char *name = "ok_exs_sendfile";

	p->file_length = 1024;

	exs_xferfile_t sendvec;
	memset(&sendvec, 0, sizeof(exs_xferfile_t));

	int fd = open(exs_test_small_file, O_RDONLY);

	sendvec.exs_xfv.exs_fdv.exs_fildes = fd;
	sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
	sendvec.exs_xfv.exs_fdv.exs_length = (size_t) 0;	//go until EOF
	sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

	p->sendvec = &sendvec;
	p->sendvec->exs_xfv_type = EXS_FDVEC;

	p->sendvec_cnt = 1;

	int ret =
	    exs_sendfile(p->fd, p->sendvec, p->sendvec_cnt, p->flags,
			 p->qh, p->ah);

	close(fd);

	return ret;
}

static void start_sendfile(){
	FILE *fp = fopen(exs_test_small_file, "w+");
	ftruncate(fileno(fp), 1024);
	fclose(fp);

	fp = fopen(exs_test_large_file, "w+");
	ftruncate(fileno(fp), EXS_MAX_SENDFILE_SIZE + 1);
	fclose(fp);
}

static void end_sendfile(){
	remove(exs_test_small_file);
	remove(exs_test_large_file);
}

#endif
