/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_SERVER_SETUP_H
#define _EXS_SERVER_SETUP_H

#include "../result.h"
#include "exs_init_setup.h"
#include "exs_socket_setup.h"
#include "exs_connect_setup.h"
#include "exs_bind_setup.h"
#include "exs_listen_setup.h"
#include "exs_accept_setup.h"
#include "exs_close_setup.h"
#include "exs_mregister_setup.h"
#include "exs_qdequeue_setup.h"
#include "exs_qcreate_setup.h"
#include "exs_recv_setup.h"
#include "exs_send_setup.h"

static int blocking_server(param * p)
{
	if (p->mode == 0) {
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_socket(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_bind(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_listen(p) < 0)
			exit(EXIT_FAILURE);

		int fd = open(exs_test_fifo_1, O_WRONLY);

		if (ok_exs_accept(p) < 0)
			exit(EXIT_FAILURE);

		close(fd);

		if (ok_exs_close(p) < 0)
			exit(EXIT_FAILURE);

		fd = open(exs_test_fifo_2, O_RDONLY);

		close(fd);
	}
}

static int non_blocking_server(param * p)
{
	if (ok_exs_init(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_qcreate(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_socket(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_bind(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_listen(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_accept(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_mregister(p) < 0)
		exit(EXIT_FAILURE);
	if (ok_exs_qdequeue(p) < 0)
		exit(EXIT_FAILURE);
}

static int exs_server(param * p)
{
	if (p->mode == 0) {
		blocking_server(p);
	}
	if (p->mode > 0) {
		non_blocking_server(p);
	}
}

#endif
