/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_SHUTDOWN_SETUP_H
#define _EXS_SHUTDOWN_SETUP_H

#include "../result.h"

#define SHUTDOWN_TESTS 5

static char *all_shutdown_tests[SHUTDOWN_TESTS] = {
	"1  : EPERM                   : exs_shutdown(fd, SHUT_RDWR, flags, qh, ah) [no exs_init]",
	"2  : EBADF                   : exs_shutdown(BAD_FD, SHUT_RDWR, flags, qh, ah)",
	"3  : EINVAL                  : exs_shutdown(fd, SHUT_RDWR, BAD_FLAG, qh, ah)",
	"4  : EINVAL                  : exs_shutdown(fd, SHUT_RDWR, flags, INVALID_QHANDLE, ah)",
	"5  : OK_EXS_SHUTDOWN         : exs_shutdown(fd, SHUT_RDWR, flags, qh, ah)"
};

static int list_shutdown_tests(int select)
{
	if (select < 0 || select > SHUTDOWN_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			SHUTDOWN_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < SHUTDOWN_TESTS; i++) {
			if (strcmp(all_shutdown_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_shutdown_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_shutdown_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_shutdown(param * p)
{
	char *name = "ok_exs_shutdown";

	int ret = exs_shutdown(p->fd, SHUT_RDWR, p->flags, p->qh, p->ah);

	if (ret == -1) {
		print_error(name);
		return -1;
	}
	return 0;
}

#endif
