# unh-exs.spec
#
# Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
# All rights reserved.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING.GPL in the main directory of the UNH EXS source tree, or the
# BSD license, available from the file COPYING.BSD in the main directory
# of the UNH EXS source tree.
#
# This material is based upon work supported by the National Science
# Foundation under Grants No. OCI-1127228 and DGE-0913620.
#
# Any opinions, findings, and conclusions or recommendations
# expressed in this material are those of the authors and do not
# necessarily reflect the views of the National Science Foundation.

# Sample .spec file for UNH EXS RPM packages
# Builds two packages: unh-exs and unh-exs-devel
# Tested on Scientific Linux 7.2

# configure-time substitutions:
# VERSION: Actual UNH EXS version
# RPM_VERSION: RPM-ized UNH EXS version (just the X.Y.Z)
# RPM_RELEASE: RPM release (1 or 1.gitXYZ.SHA or 0.1 or 0.1.gitXYZ.SHA)

%{!?_pkgdocdir:%global _pkgdocdir %{_docdir}/%{name}-%{version}}

Name:		unh-exs
Version:	1.4.0
Release:	1%{?dist}
Summary:	An implementation of the Extended Sockets interface for RDMA

Group:		Development/Libraries
License:	BSD or GPLv2
URL:		http://www.iol.unh.edu/services/research/unh-exs
Source0:	http://www.iol.unh.edu/services/research/unh-exs/%{name}-1.4.0.tar.gz
BuildRoot:	%(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)

BuildRequires:	libibverbs-devel%{?_isa},librdmacm-devel%{?_isa}
BuildRequires:	libtool%{?_isa} >= 2.2
BuildRequires:	autoconf >= 2.63
BuildRequires:	automake >= 1.11

%description
The Extended Sockets API (ES-API) is a specification published by the
OpenGroup that defines extensions to the traditional socket API in order
to provide asynchronous I/O and also memory registration for Remote
Direct Memory Access (RDMA). These two major new features enable
programmers to take advantage of today’s multicore processors and RDMA
network hardware, such as InfiniBand, iWARP and RoCE interfaces, in a
convenient yet efficient manner.

The UNH EXS interface provides most of the features specified in the
ES-API, and provides a few additional features that give the programmer
more flexibility. For example, the programmer can choose to program with
synchronous rather than asynchronous I/O, and/or to program with or
without memory registration. The programmer can also conveniently “tune”
certain aspects of the EXS interface to take advantage of application
requirements in order to provide better performance.


%prep
%setup -q -n %{name}-1.4.0


%build
autoreconf -fi
%configure --enable-trace=full --disable-assert --disable-static --docdir=%{_pkgdocdir}
make V=1 %{?_smp_mflags}


%install
rm -rf %{buildroot}
make install DESTDIR=%{buildroot}
rm -f %{buildroot}%{_libdir}/libexs.la
install -m 644 CONTRIBUTING.md COPYING COPYING.BSD COPYING.GPL NEWS README TODO %{buildroot}%{_pkgdocdir}


%clean
rm -rf %{buildroot}


%files
%defattr(-,root,root,-)
%doc %{_pkgdocdir}/COPYING
%doc %{_pkgdocdir}/COPYING.BSD
%doc %{_pkgdocdir}/COPYING.GPL
%doc %{_pkgdocdir}/NEWS
%doc %{_pkgdocdir}/README
%doc %{_pkgdocdir}/README.exs_nc
%doc %{_pkgdocdir}/TODO
%doc %{_pkgdocdir}/trace-rules.txt
%{_bindir}/*
%{_libdir}/libexs.so.*


%post -p /sbin/ldconfig

%postun -p /sbin/ldconfig


%package devel
Summary:	An implementation of the Extended Sockets interface for RDMA
Group:		Development/Libraries
Requires:	unh-exs%{?_isa} = %{version}-%{release}
Requires:	libibverbs-devel,librdmacm-devel

%description devel
This package contains the header files needed to develop and build
applications which use the UNH EXS libraries, in addition to the source
code for example programs that use the UNH EXS libraries.

%files devel
%defattr(-,root,root,-)
%doc %{_pkgdocdir}/CONTRIBUTING.md
%doc %{_pkgdocdir}/examples/*
%doc %{_pkgdocdir}/ArchitectureOverview.txt
%doc %{_pkgdocdir}/exs-seqpacket.txt
%doc %{_pkgdocdir}/exs-stream-sync.txt
%doc %{_pkgdocdir}/overview.pdf
%doc %{_pkgdocdir}/protocol.txt
%doc %{_pkgdocdir}/stream-protocol.pdf
%doc %{_pkgdocdir}/state-diagram.pdf
%doc %{_pkgdocdir}/state-diagram.ps
%doc %{_mandir}/man3/exs_*
%{_includedir}/exs.h
%{_libdir}/libexs.so


%changelog
* Wed Dec  7 2016 Patrick MacArthur <pmacarth@iol.unh.edu> - 1.4.0-1
- spec version generated by configure
